﻿local _, addonTable = ...
local L = addonTable.L

local GridL = _G.Grid.L

local math_ceil = math.ceil
local math_max = math.max
local select = select

local GetNumSubgroupMembers = GetNumSubgroupMembers
local GetNumGroupMembers = GetNumGroupMembers
local GetRaidRosterInfo = GetRaidRosterInfo
local InCombatLockdown = InCombatLockdown


GridResizeFrames = Grid:NewModule("GridResizeFrames", "AceBucket-3.0")
local GridResizeFrames = GridResizeFrames
local GridLayout = Grid:GetModule("GridLayout")
local GridFrame = Grid:GetModule("GridFrame")

GridResizeFrames.defaultDB = {
  lastSize = nil,
  groupWidth = {
  	[1] = 60,
  	[2] = 55,
  	[3] = 75,
  	[4] = 80,
  	[5] = 60,
  	[6] = 50,
  	[7] = 40,
  	[8] = 37,
  },
}

local function getDynamicLayout(maxGroups)
	local layout = {}
  	local petlayout = {}
  	local raidSize = GetNumGroupMembers()
  	local currentGroups = {}
  	for i = 1, NUM_RAID_GROUPS, 1 do
    	currentGroups[i] = false;
  	end
  	for i = 1, maxGroups, 1 do
  		currentGroups[i] = true;
  	end
	if IsInRaid() then
    	--[[
    	for i = 1, raidSize do
      		local group = select(3, GetRaidRosterInfo(i))
      		currentGroups[group] = true
    	end
    	]]
    	local group = 1
    	for i = 1, NUM_RAID_GROUPS, 1 do
      		if (currentGroups[i]) then
        		layout[group] = {
          			groupFilter = ""..i,
        		}
        		petlayout[group] = {
          			groupFilter = ""..i,
        		}
        		group = group + 1
      		end
    	end
    	local groupString = ""
    	local first = true
    	for i = 1, NUM_RAID_GROUPS, 1 do
     		if (currentGroups[i]) then
        		if (first) then
          			groupString = groupString..i
          			first = false
        		else
          			groupString = groupString..','..i
        		end
      		end
    	end
    	petlayout[group]  = {
      	groupFilter = groupString,
      	isPetGroup = true,
      	unitsPerColumn = 5,
      	maxColumns = group - 1,
    	}
  	else
    	layout[1] = {
      		showParty = true,
      		showRaid = false,
      		sortMethod = "INDEX",
    	}
    	petlayout[1] = {
      		showParty = true,
      		showRaid = false,
      		sortMethod = "INDEX",
    	}
    	petlayout[2] = {
      		showParty = true,
      		showRaid = false,
      		sortMethod = "INDEX",

      		isPetGroup = true,
      		unitsPerColumn = 5,
      		maxColumns = 5,
    	}
  	end

	return layout, petlayout
end

local function getRaidSize()
	local maxGroups = 1
	local raidSize = GetNumGroupMembers()

	if raidSize > 0 then
    	maxGroups = ceil(raidSize / 5)
    	if IsInRaid() then
    		for i = 1, raidSize do
    			local group = select(3, GetRaidRosterInfo(i))
    			maxGroups = max(maxGroups, group)
    		end
    	end
	elseif GetNumSubgroupMembers() > 0 then
    	maxGroups = 1
  	end

  	return maxGroups
end

function GridResizeFrames:OnInitialize()
	self.super.OnInitialize(self)

	self.db.profile.lastSize = getRaidSize()
  	self:Debug("Max Groups:", self.db.profile.lastSize)
end

function GridResizeFrames:OnEnable()
  local opt = Grid.options.args
  opt["GridResizeFrames"] = {
    type = "group",
    name = "GridResizeFrames",
    desc = "GridResizeFrames",
	args = {
    	OneGroupWidth = {
			name = "Single Group Width",
			desc = "Adjust the width of each unit's frame when there is only one group.",
			order = 1, width = "double",
			type = "range", min = 10, max = 100, step = 1,
			get = function(info)
				return self.db.profile.groupWidth[1]
			end,
			set = function(info, v)
				self.db.profile.groupWidth[1] = v
				if (self.db.profile.lastSize == 1) then
					GridFrame.db.profile.frameWidth = v
					GridFrame:ResizeAllFrames()
					GridFrame:ScheduleTimer("Grid_ReloadLayout", 0.5)
				end
			end,
		},
		TwoGroupWidth = {
			name = "Two Group Width",
			desc = "Adjust the width of each unit's frame when there are two groups.",
			order = 2, width = "double",
			type = "range", min = 10, max = 100, step = 1,
			get = function(info)
				return self.db.profile.groupWidth[2]
			end,
			set = function(info, v)
				self.db.profile.groupWidth[2] = v
				if (self.db.profile.lastSize == 2) then
					GridFrame.db.profile.frameWidth = v
					GridFrame:ResizeAllFrames()
					GridFrame:ScheduleTimer("Grid_ReloadLayout", 0.5)
				end
			end,
		},
		ThreeGroupWidth = {
			name = "Three Group Width",
			desc = "Adjust the width of each unit's frame when there are three groups.",
			order = 3, width = "double",
			type = "range", min = 10, max = 100, step = 1,
			get = function(info)
				return self.db.profile.groupWidth[3]
			end,
			set = function(info, v)
				self.db.profile.groupWidth[3] = v
				if (self.db.profile.lastSize == 3) then
					GridFrame.db.profile.frameWidth = v
					GridFrame:ResizeAllFrames()
					GridFrame:ScheduleTimer("Grid_ReloadLayout", 0.5)
				end
			end,
		},
		FourGroupWidth = {
			name = "Four Group Width",
			desc = "Adjust the width of each unit's frame when there are four groups.",
			order = 4, width = "double",
			type = "range", min = 10, max = 100, step = 1,
			get = function(info)
				return self.db.profile.groupWidth[4]
			end,
			set = function(info, v)
				self.db.profile.groupWidth[4] = v
				if (self.db.profile.lastSize == 4) then
					GridFrame.db.profile.frameWidth = v
					GridFrame:ResizeAllFrames()
					GridFrame:ScheduleTimer("Grid_ReloadLayout", 0.5)
				end
			end,
		},
		FiveGroupWidth = {
			name = "Five Group Width",
			desc = "Adjust the width of each unit's frame when there are three groups.",
			order = 5, width = "double",
			type = "range", min = 10, max = 100, step = 1,
			get = function(info)
				return self.db.profile.groupWidth[5]
			end,
			set = function(info, v)
				self.db.profile.groupWidth[5] = v
				if (self.db.profile.lastSize == 5) then
					GridFrame.db.profile.frameWidth = v
					GridFrame:ResizeAllFrames()
					GridFrame:ScheduleTimer("Grid_ReloadLayout", 0.5)
				end
			end,
		},
		SixGroupWidth = {
			name = "Six Group Width",
			desc = "Adjust the width of each unit's frame when there are six groups.",
			order = 6, width = "double",
			type = "range", min = 10, max = 100, step = 1,
			get = function(info)
				return self.db.profile.groupWidth[6]
			end,
			set = function(info, v)
				self.db.profile.groupWidth[6] = v
				if (self.db.profile.lastSize == 6) then
					GridFrame.db.profile.frameWidth = v
					GridFrame:ResizeAllFrames()
					GridFrame:ScheduleTimer("Grid_ReloadLayout", 0.5)
				end
			end,
		},
		SevenGroupWidth = {
			name = "Seven Group Width",
			desc = "Adjust the width of each unit's frame when there are seven groups.",
			order = 7, width = "double",
			type = "range", min = 10, max = 100, step = 1,
			get = function(info)
				return self.db.profile.groupWidth[7]
			end,
			set = function(info, v)
				self.db.profile.groupWidth[7] = v
				if (self.db.profile.lastSize == 7) then
					GridFrame.db.profile.frameWidth = v
					GridFrame:ResizeAllFrames()
					GridFrame:ScheduleTimer("Grid_ReloadLayout", 0.5)
				end
			end,
		},
		EightGroupWidth = {
			name = "Eight Group Width",
			desc = "Adjust the width of each unit's frame when there are eight groups.",
			order = 8, width = "double",
			type = "range", min = 10, max = 100, step = 1,
			get = function(info)
				return self.db.profile.groupWidth[8]
			end,
			set = function(info, v)
				self.db.profile.groupWidth[8] = v
				if (self.db.profile.lastSize == 8) then
					GridFrame.db.profile.frameWidth = v
					GridFrame:ResizeAllFrames()
					GridFrame:ScheduleTimer("Grid_ReloadLayout", 0.5)
				end
			end,
		},
    },
  }

  self.super.OnEnable(self)

  self:RegisterBucketEvent({"GROUP_ROSTER_UPDATE", "PLAYER_ENTERING_WORLD"}, 0.5, "CheckRoster")
  self:RegisterEvent("Grid_LeavingCombat", "ScheduleUpdate")

  self.scheduleUpdate = false
end

function GridResizeFrames:OnDisable()
  self.scheduleUpdate = false
  self:UnregisterAllEvents()
end

function GridResizeFrames:ScheduleUpdate()
  if self.scheduleUpdate then
    self.scheduleUpdate = false
    self:CheckRoster()
  end
end

function GridResizeFrames:CheckRoster()
  if self == nil then self = GridResizeFrames end
  if InCombatLockdown() then
  	self.scheduleUpdate = true
  	self:Debug("In combat, waiting to update.")
  	return
  end

  local maxGroups = getRaidSize()
  local layout, petlayout = getDynamicLayout(maxGroups)
  local currentLayout = GridLayout.db.profile.layout

  GridLayout:AddLayout(L["LAYOUT_NAME"], layout)

  if (self.db.profile.lastSize ~= maxGroups) and (currentLayout == L["LAYOUT_NAME"]) then
  	self:Debug("Changed number of groups from", self.db.profile.lastSize, "to", maxGroups)
  	self.db.profile.lastSize = maxGroups
  	GridFrame.db.profile.frameWidth = self.db.profile.groupWidth[maxGroups]
  	GridFrame:ResizeAllFrames()
  	GridFrame:ScheduleTimer("Grid_ReloadLayout", 0.5)
  else
  	self:Debug("Number of groups not changing, aborting resize.")
  end

  GridLayout:ReloadLayout()

end
